@@ -3,78 +3,58 @@ extern crate syn;
 #[macro_use]
 extern crate quote;
 use proc_macro::TokenStream;
-use syn::{parse_macro_input, DeriveInput};
+use syn::parse_macro_input;
 
-/// Generate helper structs/functions for benchmark configs
-/// For example, given the following struct:
-/// ```ignore
-/// #[derive(Debug, BenchConfig, Deserialize, Serialize)]
-/// pub struct TableConfig {
-///     name: String,
-///     #[matrix]
-///     threads: usize,
-/// }
-/// ```
-/// It adds a matrix struct:
-/// ```ignore
-/// pub struct TableConfigMatrix {
-///     name: String,
-///     threads: Vec<usize>
-/// }
-/// ```
-/// The `TableConfigMatrix` allows you to write concise config in .toml files, i.e.
-/// ```toml
-/// [[table]]
-/// name = "dist"
-/// threads = [1, 2, 4, 8, 16, 24]
-/// ```
-/// It also adds the following helper functions:
-/// ```ignore
-/// impl TableConfigMatrix {
-///     fn unfold(&self) -> Vec<TableConfig>;
-///     pub fn is_match(&self, filter: &regex::Regex) -> bool;
-/// }
-/// ```
-#[proc_macro_derive(ShumaiConfig, attributes(matrix))]
-pub fn derive_bench_config(input: TokenStream) -> TokenStream {
-    let ast = parse_macro_input!(input as DeriveInput);
-    let name = ast.ident;
+#[proc_macro_attribute]
+pub fn config(args: TokenStream, input: TokenStream) -> TokenStream {
+    let args = parse_macro_input!(args as syn::AttributeArgs)
+        .first()
+        .expect("Benchmark file must be annotated with #[config(path = \"/path/to/file.toml\")]")
+        .clone();
+    let file_path = get_config_file_path(&args)
+        .expect("Benchmark file must be annotated with #[config(path = \"/path/to/file.toml\")]");
 
-    let mident = gen_matrix_name(&name);
-    let lower_name = gen_lower_ident(&name);
+    let ty: syn::Item = syn::parse_macro_input!(input as syn::Item);
 
-    let fields = if let syn::Data::Struct(syn::DataStruct {
-        fields: syn::Fields::Named(syn::FieldsNamed { ref named, .. }),
-        ..
-    }) = ast.data
+    let item_struct = if let syn::Item::Struct(m) = ty {
+        m
+    } else {
+        panic!("config attribute must be applied to a Struct");
+    };
+
+    let name = item_struct.ident.clone();
+    let matrix_name = gen_matrix_name(&name);
+
+    let fields = if let syn::Fields::Named(syn::FieldsNamed { ref named, .. }) = item_struct.fields
     {
         named
     } else {
-        unimplemented!();
+        panic!("config attribute must be applied to a Struct with named fields");
     };
 
     let config_fields = fields.iter().map(|f| {
         let name = &f.ident;
         let ty = &f.ty;
+
         if name.as_ref().unwrap() == "threads" && is_matrix_field(f) {
             panic!("threads can't be marked as matrix, it's matrix by definition");
         }
         if is_matrix_field(f) {
-            quote! { #name: std::vec::Vec<#ty> }
+            quote! {#name: std::vec::Vec<#ty>}
         } else {
-            quote! { #name: #ty }
+            quote! {#name: #ty}
         }
     });
 
     let methods = gen_methods(fields, 0, &name);
-
+    let dummy_struct_name = syn::Ident::new(&format!("{}DummyStruct", name), name.span());
     let expanded = quote! {
         #[derive(Debug, shumai::__dep::serde::Deserialize)]
-        pub struct #mident {
+        pub struct #matrix_name {
             #(#config_fields, )*
         }
 
-        impl #mident {
+        impl #matrix_name {
             pub fn unfold(&self) -> std::vec::Vec<#name> {
                 let mut configs: std::vec::Vec<#name> = std::vec::Vec::new();
 
@@ -84,18 +64,41 @@ pub fn derive_bench_config(input: TokenStream) -> TokenStream {
             }
         }
 
+        #[derive(Debug, Clone, shumai::ShumaiConfig, shumai::__dep::serde::Serialize, shumai::__dep::serde::Deserialize)]
+        #item_struct
+
+        #[derive(shumai::__dep::serde::Deserialize, Debug)]
+        #[allow(non_snake_case)]
+        struct #dummy_struct_name {
+            #name: std::option::Option<std::vec::Vec<#matrix_name>>,
+        }
+
         impl #name {
-            pub fn load_with_filter(filter: impl AsRef<str>) -> std::option::Option<std::vec::Vec<#name>> {
-                let configs = BenchRootConfig::load_config().#lower_name()?;
-
-                let regex_filter =
-                            shumai::__dep::regex::Regex::new(filter.as_ref()).expect("failed to parse the benchmark filter into regex expression!");
-                let configs: std::vec::Vec<_> = configs.into_iter().filter(|c| regex_filter.is_match(&c.name)).collect();
-                Some(configs)
-            }
-
+            #[allow(non_snake_case)]
             pub fn load() -> std::option::Option<std::vec::Vec<#name>> {
-                #name::load_with_filter(".*")
+                let contents = std::fs::read_to_string(#file_path).expect(&format!("failed to read the benchmark config file at {}", #file_path));
+                let configs = shumai::__dep::toml::from_str::<#dummy_struct_name>(&contents).expect(&format!("failed to parse the benchmark config file at {}", #file_path));
+
+                let configs = configs.#name?;
+
+                let mut expanded = std::vec::Vec::new();
+                for b in configs.iter() {
+                    expanded.extend(b.unfold());
+                }
+
+
+                match std::env::var("SHUMAI_FILTER") {
+                    Ok(filter) => {
+                        let regex_filter =
+                            shumai::__dep::regex::Regex::new(filter.as_ref())
+                            .expect(&format!("Filter {} from env `SHUMAI_FILTER` is not a valid regex expression!", filter));
+                        let configs: std::vec::Vec<_> = expanded.into_iter().filter(|c| regex_filter.is_match(&c.name)).collect();
+                        Some(configs)
+                    },
+                    Err(_) => {
+                        Some(expanded)
+                    }
+                }
             }
         }
 
@@ -114,98 +117,13 @@ pub fn derive_bench_config(input: TokenStream) -> TokenStream {
         }
     };
 
+    // eprintln!("{}", expanded);
     expanded.into()
 }
 
-#[proc_macro_attribute]
-pub fn config(args: TokenStream, input: TokenStream) -> TokenStream {
-    let args = parse_macro_input!(args as syn::AttributeArgs)
-        .first()
-        .expect("Benchmark file must be annotated with #[config(path = \"/path/to/file.toml\")]")
-        .clone();
-    let file_path = get_config_file_path(&args)
-        .expect("Benchmark file must be annotated with #[config(path = \"/path/to/file.toml\")]");
-
-    let ty: syn::Item = syn::parse_macro_input!(input as syn::Item);
-
-    let mut all_configs = Vec::new();
-
-    let item_mod = if let syn::Item::Mod(m) = ty {
-        m
-    } else {
-        panic!("config attribute must be applied to mod");
-    };
-
-    let mut mod_imports = Vec::new();
-    let mod_items = item_mod.content.expect("mod can't be empty").1;
-    for item in mod_items.iter() {
-        match item {
-            syn::Item::Struct(conf_s) => all_configs.push(conf_s),
-            syn::Item::Use(u) => mod_imports.push(u),
-            _ => {}
-        }
-    }
-
-    let mod_name = item_mod.ident;
-
-    let root_fields = all_configs.iter().map(|f| {
-        let matrix_name = gen_matrix_name(&f.ident);
-        let lower_name = gen_lower_ident(&f.ident);
-        quote! {
-            #lower_name: std::option::Option<std::vec::Vec<#matrix_name>>
-        }
-    });
-
-    let field_functions = all_configs.iter().map(|f| {
-        let ident = &f.ident;
-        let lower_name = gen_lower_ident(ident);
-        quote! {
-            pub fn #lower_name(&self) -> std::option::Option<std::vec::Vec<#ident>>{
-                let mut configs = std::vec::Vec::new();
-                for b in self.#lower_name.as_ref()?.iter(){
-                    configs.extend(b.unfold());
-                }
-
-                Some(configs)
-            }
-        }
-    });
-
-    let all_config_derived = all_configs.iter().map(|s| {
-        quote! {
-            #[derive(shumai::ShumaiConfig, shumai::__dep::serde::Serialize, shumai::__dep::serde::Deserialize, Clone, Debug)]
-            #s
-        }
-    });
-
-    let mod_vis = item_mod.vis;
-    let root_config = quote! {
-        #mod_vis mod #mod_name {
-            #(#mod_imports)*
-
-            #(#all_config_derived)*
-
-            #[derive(shumai::__dep::serde::Deserialize, Debug)]
-            struct BenchRootConfig {
-                #(#root_fields,)*
-            }
-
-            impl BenchRootConfig {
-                fn load_config()-> Self {
-                    let contents = std::fs::read_to_string(#file_path).unwrap();
-                    let configs: BenchRootConfig =
-                        shumai::__dep::toml::from_str(&contents).expect("Unable to parse the toml file");
-                    configs
-                }
-
-                #(#field_functions)*
-            }
-        }
-    };
-
-    // eprintln!("{}", root_config);
-
-    root_config.into()
+#[proc_macro_derive(ShumaiConfig, attributes(matrix))]
+pub fn derive_bench_config(_input: TokenStream) -> TokenStream {
+    quote!().into()
 }
 
 fn gen_matrix_name(name: &syn::Ident) -> syn::Ident {
@@ -229,7 +147,7 @@ fn gen_methods(
                 name_gen = quote! {
                     #name_gen
                     if self.#f_name.len() > 1 {
-                        name_lit = format!("{}-{}", name_lit, #f_name);
+                        name_lit = format!("{}-{:?}", name_lit, #f_name);
                     }
                 }
             }
@@ -263,7 +181,7 @@ fn gen_methods(
     if is_matrix_field(current) {
         quote! {
             for i in self.#name.iter() {
-                let #name = *i;
+                let #name = i.clone();
                 #inner
             }
         }
@@ -284,11 +202,6 @@ fn is_matrix_field(f: &syn::Field) -> bool {
     false
 }
 
-fn gen_lower_ident(f: &syn::Ident) -> syn::Ident {
-    let lower_name = f.to_string().to_ascii_lowercase();
-    syn::Ident::new(&lower_name, f.span())
-}
-
 fn get_config_file_path(meta: &syn::NestedMeta) -> Option<syn::LitStr> {
     let meta = if let syn::NestedMeta::Meta(m) = meta {
         m
